@@ -49,7 +49,7 @@ struct _GtkMineField {
 	gint cdown;
         guint cdownx;
         guint cdowny;
-	gint bdown;
+	gint bdown[3];
         gint lose;
         gint win;
 	gint multi_mode;
